import { METHOD, initRoutes, getPath, getRoute, pathOptions, joinResponseRoutes } from './RouteCreator';
import { MockOptions } from './AutoMockServer';
import { OpenAPI } from 'openapi-types';
import { openApiToMockFile } from './OpenApiConverter';
declare const PATH_SYMBOLS: {
    BIND: (dKey: any, rKey: any) => string;
    PROTOCOL: symbol;
    DYNAMIC: symbol;
    ROUTE: symbol;
    NAME: symbol;
    QUERY: symbol;
    BODY: symbol;
    RESPONSE: symbol;
    INDEX: symbol;
};
declare let AutoMock: {
    getMockData?: (path: string, protocol: string) => any;
    setMockData?: (path: string, protocol: string, data: any, filter: (reqData: any, req: Request) => any) => void;
    mock?: any;
};
declare const getMockData: (path: string, protocol: string) => any;
declare const setMockData: (path: string, protocol: string, data: any, filter: (reqData: any, req: Request) => any) => void;
declare let RouteCreator: {
    METHOD: Record<"GET" | "POST" | "PUT" | "DELETE" | "PATCH" | "TRACE" | "HEAD" | "OPTIONS", "GET" | "POST" | "PUT" | "DELETE" | "PATCH" | "TRACE" | "HEAD" | "OPTIONS">;
    SYMBOLS: {
        PROTOCOL: symbol;
        DYNAMIC: symbol;
        ROUTE: symbol;
        NAME: symbol;
        QUERY: symbol;
        BODY: symbol;
        RESPONSE: symbol;
        INDEX: symbol;
    };
    initRoutes: typeof initRoutes;
    getPath: typeof getPath;
    getRoute: typeof getRoute;
    pathOptions: typeof pathOptions;
    joinResponseRoutes: typeof joinResponseRoutes;
    BIND: (dKey: any, rKey: any) => string;
};
/**
 * Creates a mock server out of an API document, also enables adding external mock reference object.
 * @param document Open API document to convert
 * @param mockRef mock references object to attach to the open API document, this can be done in two ways
 * adding a key corresponding to an operation id, with a value of a mock [RESPONSE] object i.e.:
 * { template: {a:"[1|2|3]"} }
 * adding a key corresponding to a x-mock-ref key on a property of the open api mock,
 * with a value of a mock template i.e.:
 * {a:"[1|2|3]"}
 * @param options mock options
 */
declare function mockOpenApi(document: string | OpenAPI.Document, mockRef?: object, options?: MockOptions): void;
declare function mock(routes: Object, options?: MockOptions): void;
export { RouteCreator, AutoMock, PATH_SYMBOLS, METHOD, mock, mockOpenApi, openApiToMockFile, getMockData, setMockData, };
